package com.iotechn.unimall.admin.api.cloud;


import com.iotechn.unimall.data.domain.ArticleManagement;
import com.iotechn.unimall.core.annotation.HttpMethod;
import com.iotechn.unimall.core.annotation.HttpOpenApi;
import com.iotechn.unimall.core.annotation.HttpParam;
import com.iotechn.unimall.core.annotation.HttpParamType;
import com.iotechn.unimall.core.annotation.param.NotNull;
import com.iotechn.unimall.core.exception.ServiceException;
import com.iotechn.unimall.data.model.Page;
import java.util.Date;

/**
 * 文章管理信息Service接口
 * 
 * @author jlb
 * @date 2023-05-26
 */
@HttpOpenApi(group = "admin.unimall.articleManagement", description = "文章管理信息")
public interface IArticleManagementService{
	@HttpMethod(description = "新增",  permissionName = "文章管理信息管理")
	public Boolean add(@NotNull @HttpParam(name = "articleManagement", type = HttpParamType.COMMON, description = "文章管理信息") ArticleManagement articleManagement)throws ServiceException;

	@HttpMethod(description = "列表",  permissionName = "文章管理信息管理")
	public Page<ArticleManagement> list(
								@HttpParam(name = "articleTitle", type = HttpParamType.COMMON, description = "文章标题") String articleTitle,
							@HttpParam(name = "articleIntroduction", type = HttpParamType.COMMON, description = "文章简介") String articleIntroduction,
							@HttpParam(name = "articleClassify", type = HttpParamType.COMMON, description = "文章分类") String articleClassify,
							@HttpParam(name = "coverImage", type = HttpParamType.COMMON, description = "封面") String coverImage,
							@HttpParam(name = "classification", type = HttpParamType.COMMON, description = "分类") String classification,
							@HttpParam(name = "articleTemplate", type = HttpParamType.COMMON, description = "文章模板") String articleTemplate,
							@HttpParam(name = "searchKeywords", type = HttpParamType.COMMON, description = " 搜索关键字") String searchKeywords,
							@HttpParam(name = "language", type = HttpParamType.COMMON, description = " 语言") String language,
							@HttpParam(name = "status", type = HttpParamType.COMMON, description = "状态") String status,
							@HttpParam(name = "gmtCreate", type = HttpParamType.COMMON, description = "") Date gmtCreate,
							@HttpParam(name = "gmtUpdate", type = HttpParamType.COMMON, description = "") Date gmtUpdate,
							@HttpParam(name = "deleteFlag", type = HttpParamType.COMMON, description = "删除标识") Long deleteFlag,
					@HttpParam(name = "page", type = HttpParamType.COMMON, description = "页码", valueDef = "1") Integer page,
		@HttpParam(name = "limit", type = HttpParamType.COMMON, description = "页码长度", valueDef = "20") Integer limit)
		throws ServiceException;

	@HttpMethod(description = "删除", permissionName = "文章管理信息管理")
	public Boolean delete(@NotNull @HttpParam(name = "id", type = HttpParamType.COMMON, description = "")String id)throws ServiceException;

	@HttpMethod(description = "修改", permissionName = "文章管理信息管理")
	public Boolean update(@NotNull @HttpParam(name = "articleManagement", type = HttpParamType.COMMON, description = "文章管理信息") ArticleManagement articleManagement)throws ServiceException;

	@HttpMethod(description = "查询",permissionName = "文章管理信息管理")
	public ArticleManagement get(@NotNull @HttpParam(name = "id", type = HttpParamType.COMMON, description = "")Long id)throws ServiceException;
	
	@HttpMethod(description = "导出excl表",  permissionName = "文章管理信息管理")
	public String export(
								@HttpParam(name = "articleTitle", type = HttpParamType.COMMON, description = "文章标题") String articleTitle,
							@HttpParam(name = "articleIntroduction", type = HttpParamType.COMMON, description = "文章简介") String articleIntroduction,
							@HttpParam(name = "coverImage", type = HttpParamType.COMMON, description = "封面") String coverImage,
							@HttpParam(name = "classification", type = HttpParamType.COMMON, description = "分类") String classification,
							@HttpParam(name = "articleTemplate", type = HttpParamType.COMMON, description = "文章模板") String articleTemplate,
							@HttpParam(name = "status", type = HttpParamType.COMMON, description = "状态") String status,
							@HttpParam(name = "gmtCreate", type = HttpParamType.COMMON, description = "") Date gmtCreate,
							@HttpParam(name = "gmtUpdate", type = HttpParamType.COMMON, description = "") Date gmtUpdate,
							@HttpParam(name = "deleteFlag", type = HttpParamType.COMMON, description = "删除标识") Long deleteFlag,
				@HttpParam(name = "page", type = HttpParamType.COMMON, description = "页码", valueDef = "1") Integer page,
	@HttpParam(name = "limit", type = HttpParamType.COMMON, description = "页码长度", valueDef = "20") Integer limit)throws ServiceException;
	
}
